/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : cfg_addr_decode.v
   Created on           : 19 Feb 1999
   Function             : Generates register selects for config registers
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Mar 3rd 99		      Name changes for coding guidelines 
   Mar 4th 99			  Added interrupt register 

   Instantiated in      : tgt_addr_ctl

   Instantiates         : no lower level blocks
   Synthesis check      : not done 
=========================================================================*/

/* The I_cfg_rd and I_cfg_wr bits from the target command decoder have to be 
   validated by AD[1:0] = 00 - indicating linear addressing mode - must 
   for configuration space transfers.and AD[10:0] = 000 - indicating a 
   single function PCI device along with the assertion of IDSEL for making
   access to configuration space possible. The address bits are taken care
   of in the addr_latch block and generate a cfg_spacer_enable bit. IDSEL 
   is taken care of at the command decoder itself. This block generates the 
   O_valid_cfg_wr and O_valid_cfg_rd which go the the cfg_space block as 
   the write and read enables.
*/

module cfg_addr_decode (

					I_RST_N ,
                    I_CLK,

					I_cfg_rd, 			// From target command decoder
					I_cfg_wr,
					I_cfg_spc_enb,	// From addr_latch block 

					I_reg_addr,			// from latched_address_lo [7:2] 
   	     
					O_valid_cfg_rd,     
					O_valid_cfg_wr,     
                    O_device_reg_sel,
                    O_cs_reg_sel,
                    O_cc_reg_sel,
                    O_lt_reg_sel,
                    O_bar0_lo_reg_sel,
                    O_bar0_hi_reg_sel,
                    O_bar1_reg_sel,
					O_intr_reg_sel
);

input   I_RST_N ;
input   I_CLK ;

input	I_cfg_rd ; 
input	I_cfg_wr ;
input 	I_cfg_spc_enb ;
input	[7:2] I_reg_addr;

output 	O_valid_cfg_rd ;
output 	O_valid_cfg_wr ;

output  O_device_reg_sel ;
output  O_cs_reg_sel;
output  O_cc_reg_sel ;
output  O_lt_reg_sel ;
output  O_bar0_lo_reg_sel ;
output  O_bar0_hi_reg_sel ;
output  O_bar1_reg_sel ;
output  O_intr_reg_sel ;

reg [7:0] decoded_vector ;
wire decode_enable ;

assign O_valid_cfg_rd  = I_cfg_rd & I_cfg_spc_enb ;
assign O_valid_cfg_wr  = I_cfg_wr & I_cfg_spc_enb ;

assign decode_enable = O_valid_cfg_rd | O_valid_cfg_wr ;
assign O_device_reg_sel = decoded_vector[0];
assign O_cs_reg_sel = decoded_vector[1];
assign O_cc_reg_sel = decoded_vector[2];
assign O_lt_reg_sel = decoded_vector[3];
assign O_bar0_lo_reg_sel = decoded_vector[4];
assign O_bar0_hi_reg_sel = decoded_vector[5];
assign O_bar1_reg_sel = decoded_vector[6];
assign O_intr_reg_sel = decoded_vector[7];

always@(negedge I_RST_N or posedge I_CLK )
begin

if (!I_RST_N)
   decoded_vector = 7'b0000000 ;
  
else  
   if (decode_enable)
   begin 
      case (I_reg_addr)
      0 : decoded_vector = 8'b00000001 ;
      1 : decoded_vector = 8'b00000010 ;
      2 : decoded_vector = 8'b00000100 ;
      3 : decoded_vector = 8'b00001000 ;
      4 : decoded_vector = 8'b00010000 ;
      5 : decoded_vector = 8'b00100000 ;
      6 : decoded_vector = 8'b01000000 ;
      15: decoded_vector = 8'b10000000 ; // corresponds to reg address 3Ch
      default : decoded_vector = 7'b0000000 ;
      endcase 
   end 
end  

endmodule 

