/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : cfg_space.v
   Created on           : 16 Feb 1999
   Function             : This is a structural model of the configuratuion 
                          space registers in CLPCI
   Author               : Ramaprasad K V 
   Revision             : 1.0
   Verified             : no
   Modification History : 

   2 March 				  signal naming changed according to coding guidelines
   4 Mar 99			      Output mux included 
   4 Mar 99               Writes to registers validated with TRDY 

   Instantiated in      : tgt_top.v


   Instantiates         : device_reg.v 
       					  sts_cmd_reg.v
						  class_code_reg.v
						  lat_timer_reg.v
						  bar0_lo_reg.v
						  bar0_hi_reg.v
						  bar1_reg.v
						  regout_mux.v

   Synthesis check      : done 
=========================================================================*/

// This module implements the configuration space registers of CLPCI
// This is purely a structural implementation

`include "header.h"

module cfg_space (
                    I_in_data_bus,
                    I_CLK,
                    I_RST_N,

                    I_wr_enb,
                    I_rd_enb,
					I_trdy_N,

                    I_device_reg_sel,  			// Register selects from Tgt_addr_ctl 
                    I_cs_reg_sel,			
                    I_cc_reg_sel,		
                    I_lt_reg_sel,		
                    I_bar0_lo_reg_sel,
                    I_bar0_hi_reg_sel,
                    I_bar1_reg_sel,
                    I_intr_reg_sel,

                    I_set_prty_err_reported,
                    I_set_detected_prty_err,    

                    I_set_signalled_tgt_abort ,

                    I_set_recd_tgt_abort,     
                    I_set_recd_mstr_abort, 
                    I_set_signalled_serr,   

                    O_io_spc_enb,     
                    O_mem_spc_enb,  
                    
                    O_prty_err_resp, 
                    O_master_enb,    
                    O_serr_enb,     
                    O_lat_timer_val,

                    O_mem_base_addr,
                    O_io_base_addr, 
                    
                    O_out_data_bus

 );

input [31:0] I_in_data_bus ;

input I_CLK ;
input I_RST_N ;

input I_wr_enb ;                 // from target command decoder
input I_rd_enb ;                 //          "
input I_trdy_N ;

input I_device_reg_sel ;     
input I_cs_reg_sel ;        
input I_cc_reg_sel ;        
input I_lt_reg_sel ;        
input I_bar0_lo_reg_sel ;   
input I_bar0_hi_reg_sel ;   
input I_bar1_reg_sel ;      
input I_intr_reg_sel ;      

input I_set_prty_err_reported; // from parity gen block
input I_set_detected_prty_err;      // from parity check block

input I_set_signalled_tgt_abort ;   // from target engine

input I_set_recd_tgt_abort;         // from initiator engine
input I_set_recd_mstr_abort;      //          "
input I_set_signalled_serr;         //          "

output O_io_spc_enb;           // to target command decoder
output O_mem_spc_enb;          //          " 

output O_prty_err_resp;         // to initiator & target engine
output O_master_enb;             // to initiator engine
output O_serr_enb;               //          " 
output [7:0] O_lat_timer_val;   //          "

output [63:`mem_addr_width] O_mem_base_addr;   // to target command decoder
output [31:8] O_io_base_addr;       //          "                Lower 8 bits not needed for decode

output  [31:0] O_out_data_bus ;


wire O_io_spc_enb;           
wire O_mem_spc_enb;          
wire O_master_enb;             
wire O_prty_err_resp;         
wire O_serr_enb;               
wire [7:0] O_lat_timer_val;   
wire [63:`mem_addr_width] O_mem_base_addr;          
wire [31:8] O_io_base_addr;              

wire [31:0] device_reg_out ;
wire [31:0] cmd_sts_reg_out ;
wire [31:0] class_code_reg_out ;
wire [31:0] lat_timer_reg_out ;
wire [31:0] bar0_lo_reg_out ;
wire [31:0] bar0_hi_reg_out ;
wire [31:0] bar1_reg_out ;
wire [31:0] intr_reg_out ;

wire wr_enb_to_reg ;

assign wr_enb_to_reg  = ~I_trdy_N & I_wr_enb ; // I_wr_enb should be asserted with
										       // trdy_N asserted for writes tp happen
device_reg device_reg (

                    .O_data_out (device_reg_out),
                    .I_reg_sel  (I_device_reg_sel),
                    .I_rd_enb   (I_rd_enb) 
);


cmd_sts_reg cmd_sts_reg (
                     I_CLK,
                     I_RST_N,
                     I_in_data_bus,
                     I_cs_reg_sel,
					 wr_enb_to_reg,
					 I_rd_enb,
                     I_set_prty_err_reported, // bit 8
                     set_signalled_system_error, // bit 11
                     set_received_master_abort,  // bit 12
                     set_received_target_abort,  // bit 13
                     set_signalled_target_abort, // bit 14
                     set_data_prty_err_detected, // bit 15

                     cmd_sts_reg_out,
                     O_io_spc_enb,     
                     O_mem_spc_enb,  
                     O_prty_err_resp, 
                     O_master_enb,    
                     O_serr_enb     
);

class_code_reg cc_reg_inst (
                    
                    .O_data_out (class_code_reg_out),
                    .I_reg_sel (I_cc_reg_sel),
                    .I_rd_enb (I_rd_enb) 
              );


lat_timer_reg lat_timer_reg (

                       .I_RST_N (I_RST_N),
                       .I_CLK (I_CLK),
                       .I_reg_sel (I_lt_reg_sel),
                       .I_in_data (I_in_data_bus),
                       .I_wr_enb (wr_enb_to_reg),
                       .O_timer_val (O_lat_timer_val),
                       .O_out_data (lat_timer_reg_out)

);



bar0_lo_reg  mem_lo_addr ( 
                    .I_CLK (I_CLK),
                    .I_RST_N (I_RST_N),
                    .I_in_data (I_in_data_bus),
                    .I_reg_sel (I_bar0_lo_reg_sel),
                    .I_wr_enb (wr_enb_to_reg),
                    .I_rd_enb (I_rd_enb),
                    .O_out_data (bar0_lo_reg_out),
                    .O_mem_base_addr_lo (O_mem_base_addr[31:`mem_addr_width])
);

bar0_hi_reg  mem_hi_addr ( 
                    .I_CLK  (I_CLK), 
                    .I_RST_N  (I_RST_N),
                    .I_in_data (I_in_data_bus),
                    .I_reg_sel (I_bar0_hi_reg_sel),
                    .I_wr_enb (wr_enb_to_reg),
                    .I_rd_enb (I_rd_enb), 
                    .O_out_data  (bar0_hi_reg_out),
                    .O_mem_base_addr_hi (O_mem_base_addr[63:32])
);


bar1_reg io_base_reg ( 

                    .I_CLK  		(I_CLK),
                    .I_RST_N  		(I_RST_N),
                    .I_in_data  	(I_in_data_bus),
                    .I_reg_sel 		(I_bar1_reg_sel),
                    .I_wr_enb 		(wr_enb_to_reg),
                    .I_rd_enb 		(I_rd_enb),
                    .O_out_data 	(bar1_reg_out),
                    .O_io_base_addr (O_io_base_addr)
);



intr_reg intr_reg (

                       .I_RST_N		(I_RST_N),
                       .I_CLK		(I_CLK),
                       .I_reg_sel	(I_intr_reg_sel),
                       .I_in_data	(I_in_data_bus),
                       .I_wr_enb	(wr_enb_to_reg),
                       .O_out_data	(intr_reg_out)

);


regout_mux  regout_mux (

					.I_device_reg_out  		(device_reg_out) ,  	
					.I_cmd_sts_reg_out  	(cmd_sts_reg_out) ,
                    .I_class_code_reg_out	(class_code_reg_out) ,
					.I_lat_timer_reg_out 	(lat_timer_reg_out)  ,
					.I_bar0_lo_reg_out 		(bar0_lo_reg_out)  ,
					.I_bar0_hi_reg_out 		(bar0_hi_reg_out ) ,
					.I_bar1_reg_out 		(bar1_reg_out),
					.I_intr_reg_out			(intr_reg_out) ,
					.I_rd_enb				(I_rd_enb),

                    .I_device_reg_sel		(I_device_reg_sel),  
                    .I_cs_reg_sel			(I_cs_reg_sel),		
                    .I_cc_reg_sel			(I_cc_reg_sel),		
                    .I_lt_reg_sel			(I_lt_reg_sel),		
                    .I_bar0_lo_reg_sel		(I_bar0_lo_reg_sel),
                    .I_bar0_hi_reg_sel		(I_bar0_hi_reg_sel),
                    .I_bar1_reg_sel			(I_bar1_reg_sel),
	 				.I_intr_reg_sel			(I_intr_reg_sel),
					.O_cfg_regs_data		(O_out_data_bus) 

);

endmodule
