/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_sm.v
   Created on           : 1 Mar 1999
   Function             : Implements the target state machine 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   3 MAr 99				  O_end_xfer signal added 
   12 Mar 99              O_read_mode added 
                          Conditions in generating oe_trdy and oe_stop in 
                          wait states added 
                          Conditions modified for generating trdy in RD 
                          states & fifo going empty
   14 Mar 99              incr_wrptr conditions modified in WR_XFER state
   15 Mar 99              Disconnect for improper burst order implemented

   Instantiated in      : tgt_engine.v

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/
/*
   The state machine has the following states

   IDLE : This is the default state. The s/m goes out of this state only
          when it detects a new target command intended for this target.
          Otherwise it continues to stay in the same state. 

          Whenever a transaction ends (either by master termination or 
          targte termination, the s/m reaches this state.
     
          In this state the target does not drive any signals on the bus
 
          when there is a new write command, the s/m goes to CHK_WRFIFO
          state. When there is a new read command , s/m goes to 
          START_RD_XFER state.

   CHK_WRFIFO :

          When the s/m is in this state, it looks if there is space in the
          target write fifo. If there is space, it will transit to WR_XFER.
          If not it will remain in this state and at each clock edge 
          increments the initial latency counter. When the counter reaches
          12, the s/m goes to the DISCONNECT state.

   WR_XFER :
          
          When s/m is in this state,  one word is written into the FIFO 
          and the FIFO pointer is incremented at each clock edge when 
          the FIFO is not full and both irdy and trdy is asserted. If there
          is a end of transfer indication (normal master termonation) 
          s/m goes to LAST_CYCLE state. If there is a abort indication
          from the back end, s/m goes to ABORT state. If there is a 
          disconnect indication, next state will be DISCONNECT. If either 
          of irdy or trdy are de-asserted or the FIFO is full, the next
          state is WR_WAIT 

   WR_WAIT :
           
          This indicates that wait states are being inserted in write 
          transfers. The next state will be WR_XFER if irdy & trdy are 
          asserted and target write fifo is not full. Otherwise the s/m 
          will remain in the same state. At each cycle when the s/m 
          is in this state a latency counter for 8 gets incremented.
          If the value reaches 8 (or 6) next state will be DISCONNECT.


   START_RD_XFER :
          
          Whenever a new read command meant for the target is detected,
          the s/m comes to this state.In this state, the request is 
          given to the back end interface. In the next clock, the s/m
          goes to RD_INIT_LAT state.

   RD_INIT_LAT :
          This denotes the initial latency time for reads. While in this
          state, the s/m waits for data to be available in the target
          read fifo. The latency timer counter increments evey clock when 
          the s/m is in this state. If data is not available till the 
          counter reaches the threshold value, the s/m goes to disconnect
          state. If data becomes available before that, it goes to 
          RD_XFER state.

   RD_XFER :

          When s/m is in this state,  one word is read from target read FIFO 
          and the FIFO read pointer is incremented at each clock edge when 
          the FIFO is not empty and both irdy and trdy is asserted. If there
          is a end of transfer indication (normal master termonation) 
          s/m goes to LAST_CYCLE state. If there is an abort indication
          from the back end, s/m goes to ABORT state. If there is a 
          disconnect indication, next state will be DISCONNECT. If either 
          of irdy or trdy are de-asserted or the FIFO is empty, the next
          state is RD_WAIT 

   RD_WAIT :
           
          This indicates the wait states being inserted in between read 
          transfers. The next state will be RD_XFER if irdy & trdy are 
          asserted and target read fifo is not empty. Otherwise the s/m 
          will remain in the same state. At each cycle when the s/m 
          is in this state a latency counter for 8 gets incremented.
          If the value reaches 8 (or 6) next state will be DISCONNECT.

   DISCONNECT:
           This indicates a disconnect on the PCI bus. 

   ABORT :
           This indicates a abort on the PCI bus. 

   LAST_CYC :

   last_transfer_flag gets set (asynchronous set) when it detects frame
   deasserted and IRDY asserted. This gets reset at a clock cycle when 
   both IRDY and TRDY are asserted. This generation is don outside this 
   block 
*/

module tgt_sm (

				I_RST_N,
				I_CLK,

				I_devsel_N,      // from addr controller, same as O_DEVSEL_n active low
                I_frame_N,		 // will be the lached version , not dirct from the bus
                I_new_tgt_cmd,
				I_tgt_wr,
				I_tgt_rd,

				I_disc_from_app,
				I_abort_from_app,
				I_do_disc,
			
				I_irdy_N,

				I_twf_full,				// target write FIFO full
				I_last_xfer_flag,
				I_single_cyc_xfer,

				I_trf_empty,			// target read FIFO empty

				O_trdy_N,	
				O_oe_trdy,
				O_stop_N,
                O_oe_stop,
  
                O_read_mode,           // for application side use

				O_oe_AD_lo  ,
				O_oe_AD_hi  ,
                O_oe_PAR ,              // Should follow data after 1 cycle
                O_oe_PAR64 ,            //                "
				O_incr_twf_wrptr,
				O_incr_trf_rdptr,

				O_end_xfer,
				O_last_cyc_indi
				
); 

parameter  IDLE           = 4'b0000 ;
parameter  CHK_WRFIFO     = 4'b0001 ;
parameter  WR_XFER        = 4'b0011 ;
parameter  WR_WAIT        = 4'b0010 ;
parameter  START_RD_XFER  = 4'b1000 ;
parameter  RD_INIT_LAT    = 4'b1001 ;
parameter  RD_XFER        = 4'b1011 ;
parameter  RD_WAIT        = 4'b1010 ;   
parameter  DISCONNECT     = 4'b1110 ;   
parameter  ABORT          = 4'b1101 ;
parameter  LAST_CYC       = 4'b1111 ;

input I_RST_N;
input I_CLK;
inout I_frame_N ;
input I_devsel_N ;
input I_new_tgt_cmd ;
input I_tgt_wr ;
input I_tgt_rd ;
input I_disc_from_app ;
input I_do_disc ;
input I_abort_from_app ;
input I_irdy_N ;
input I_twf_full ;
input I_trf_empty ;
input I_last_xfer_flag ;
inout I_single_cyc_xfer ;

output O_trdy_N ;    
output O_oe_trdy ;    
output O_stop_N ;    
output O_oe_stop ;    
output O_oe_AD_lo ;
output O_oe_PAR ;
output O_oe_AD_hi ;
output O_oe_PAR64 ;

output O_read_mode ;
output O_incr_twf_wrptr ;
output O_incr_trf_rdptr ;

output O_end_xfer ;
output O_last_cyc_indi ;

reg end_xfer_flag  ;
reg O_read_mode  ;


reg [3:0] present_state ;
reg [3:0] next_state ; 
reg trdy_N ; // internal sig
reg stop_N ;
reg O_oe_trdy ;
reg O_oe_stop ;
reg O_incr_twf_wrptr ;
reg O_incr_trf_rdptr ;

reg oe_AD_lo ;
reg O_oe_PAR ;
reg oe_AD_hi ;
reg O_oe_PAR64 ;
reg [3:0] cnt16 ; // use for latency timers 
reg [2:0] cnt8 ;

reg last_cyc_indi ;

assign O_stop_N = stop_N ;
assign O_trdy_N = trdy_N ;
assign O_oe_AD_lo = oe_AD_lo ;
assign O_oe_AD_hi = oe_AD_hi ;

always@(negedge I_RST_N or posedge I_CLK)
begin

if (I_RST_N == 1'b0) 
   begin 
      present_state <=  IDLE ;
   end 
else
   begin 
      present_state <= next_state ;
   end 
end 

assign O_end_xfer = end_xfer_flag ;//   Use to de assert devsel 
assign O_last_cyc_indi = last_cyc_indi ;// used to reset the read fifo


always@(present_state or I_devsel_N or I_new_tgt_cmd or I_tgt_wr or I_tgt_rd or 
        I_twf_full or I_trf_empty or I_abort_from_app or I_disc_from_app or I_do_disc
        or I_irdy_N or I_last_xfer_flag or cnt16 or cnt8 or I_frame_N or I_single_cyc_xfer)
begin 

   end_xfer_flag <= 0 ; // might get modified later in the execution of this process

   case (present_state)

   IDLE : 

      if ((I_devsel_N == 1'b0) && (I_new_tgt_cmd == 1'b1))
//      if (I_new_tgt_cmd == 1'b1)  // sufficient condition for transition - problem with decode time

         begin 

         if (I_tgt_wr == 1'b1) 
            next_state <= CHK_WRFIFO ;
         else if (I_tgt_rd == 1'b1)
            next_state <= START_RD_XFER ;
         else 
            next_state <= IDLE ;
         end 

      else
         next_state <= IDLE ; 

   CHK_WRFIFO :
          
      if (cnt16 == 4'b1100 ) // check value later  - 12 /16 ??
         next_state <= DISCONNECT ;
      else 
        
         if (I_twf_full == 1'b1) 
            next_state <= CHK_WRFIFO ; // keep waiting till not full
         else
            next_state <= WR_XFER ;
          
   WR_XFER: 

      begin

         if ((I_last_xfer_flag == 1'b1) | (I_single_cyc_xfer))
            begin 
               next_state <= LAST_CYC ;
               end_xfer_flag <= 1'b1 ;
            end 

         else if ((I_disc_from_app == 1'b1) ||(I_do_disc) )
	          
            next_state <= DISCONNECT ;
           
         else if (I_abort_from_app ==1'b1) 

	        next_state <= ABORT ;

	     else 
            begin 
               if ((I_irdy_N == 1'b0) && (I_twf_full == 1'b0) )

                  next_state <= WR_XFER ;

               else if  ((I_irdy_N == 1'b1) || (I_twf_full == 1'b1))
              
	              next_state <= WR_WAIT ;
            end 

      end 
   
   WR_WAIT :

      if (cnt8 == 3'b101)  // check value  - now 5 clocks
         next_state <= DISCONNECT ;

      else if ((I_irdy_N == 1'b1) || (I_twf_full == 1'b1))
            
         next_state <= WR_WAIT ;
      else
         next_state <= WR_XFER ;

   ABORT :

       if (I_frame_N == 1'b1) // Frame has been de-asserted  
          begin 
             next_state <= LAST_CYC ; 
             end_xfer_flag <= 1'b1 ;
          end 
       else 
          next_state <= ABORT  ; // keep indicating the abort signals

   DISCONNECT :

       if (I_frame_N == 1'b1) // Frame has been de-asserted     
         begin 
             next_state <= LAST_CYC ; 
             end_xfer_flag <= 1'b1 ;
         end 
       else 
          next_state <= DISCONNECT  ; // keep indicating the abort signals
       

   START_RD_XFER :
      next_state <= RD_INIT_LAT ;

   RD_INIT_LAT :
      
      if (cnt16 == 4'b1100 ) // check value later  - 12 /16 ??
         next_state <= DISCONNECT ;
      else 
        
         if (I_trf_empty == 1'b1) 
            next_state <= RD_INIT_LAT ; // keep waiting till not full
         else
            next_state <= RD_XFER ;
 
   RD_XFER :

      begin

         if ((I_last_xfer_flag == 1'b1) | (I_single_cyc_xfer))
            begin 
               next_state <= LAST_CYC ;
               end_xfer_flag <= 1'b1 ;
            end 


         else if ((I_disc_from_app == 1'b1) || (I_do_disc) )
	          
            next_state <= DISCONNECT ;
           
         else if (I_abort_from_app ==1'b1) 

	        next_state <= ABORT ;

	     else 
            begin 
               if ((I_irdy_N == 1'b0) && (I_trf_empty == 1'b0) )

                  next_state <= RD_XFER ;

               else if  ((I_irdy_N == 1'b1) || (I_trf_empty == 1'b1))
              
	              next_state <= RD_WAIT ;
            end 
      end 
   
   RD_WAIT : 

      if (cnt8 == 3'b101)  // check value  - now 5 clocks
         next_state <= DISCONNECT ;

      else if ((I_irdy_N == 1'b1) || (I_trf_empty == 1'b1))
            
         next_state <= RD_WAIT ;
      else
         next_state <= RD_XFER ;

   LAST_CYC : // all outputs should be driven with their de-asserted value
      
      next_state <= IDLE ;
	  

   default : // 
 
      next_state <= IDLE ; // full case - parallel case etc required ??

   endcase 
end


// latency counters for target initial latency and in-between-data latency


always@(posedge I_CLK or negedge I_RST_N)
begin 
if (!I_RST_N)
   begin
      cnt8 <= 0 ;
      cnt16 <= 0 ;
   end 
else
   case (present_state)
  
   CHK_WRFIFO,
   RD_INIT_LAT : 
      begin 
         cnt16 <= cnt16 +1 ;
         cnt8  <= 0 ;
      end 

   WR_WAIT,
   RD_WAIT : 
      begin 
         cnt16 <= 0 ;
         cnt8  <= cnt8 +1 ;
      end 

   default : 
      begin 
         cnt16 <= 0 ;
         cnt8  <= 0 ;
      end 
   endcase 
end  

//always@(present_state)
always@(present_state or I_trf_empty or I_last_xfer_flag or I_single_cyc_xfer)
begin 
   case (present_state)

   IDLE :
      begin 
         trdy_N <= 1'b1 ;
         stop_N <= 1'b1 ;
         O_oe_trdy <= 1'b0 ;
         O_oe_stop <= 1'b0 ;

         oe_AD_lo  <= 1'b0 ; // do not drive these sognals
         oe_AD_hi  <= 1'b0 ;

         O_incr_twf_wrptr <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;
      end 


   WR_XFER :
      begin
         trdy_N <= 1'b0 ;
         stop_N <= 1'b1 ;
         O_oe_trdy <= 1'b1 ;
         O_oe_stop <= 1'b1 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;

        // write done only if irdy asserted at clock edge & pointer should be 
        // incremented only if write is done ; Since the irdy coming to this 
        // is always synchronized, we shall use the last xfer flag for this purpose 


         if ((I_last_xfer_flag ) & (!I_single_cyc_xfer)) // CHECK 
            O_incr_twf_wrptr <= 1'b0 ;
         else
            O_incr_twf_wrptr <= 1'b1 ; // CHECK
              
      end 

   RD_XFER :
      begin
         if (I_trf_empty)   // check if last xfer should be a condition
          // immediately take of the trdy,and data enables  
          //state transition will be at clock edge 
            begin 
               trdy_N <= 1'b1 ;
               stop_N <= 1'b1 ;
               O_oe_trdy <= 1'b1 ;
               O_oe_stop <= 1'b1 ;
   
               oe_AD_lo  <= 1'b0 ;
               oe_AD_hi  <= 1'b0 ;
   
               O_incr_twf_wrptr <= 1'b0 ;
               O_incr_trf_rdptr <= 1'b0 ;  /// since trf is empty
            end 

         else 
            begin 
               trdy_N <= 1'b0 ;
               stop_N <= 1'b1 ;
               O_oe_trdy <= 1'b1 ;
               O_oe_stop <= 1'b1 ;
   
               oe_AD_lo  <= 1'b1 ; // modifications required yet for 64 bit data
               oe_AD_hi  <= 1'b1 ; // probably can be assigned to !ack64_N 
   
               O_incr_twf_wrptr <= 1'b0 ;
               O_incr_trf_rdptr <= 1'b1 ;
            end 
      end 

   DISCONNECT :  // Always  indicate disconnect without data !!
      begin

         trdy_N <= 1'b1 ;
         stop_N <= 1'b0 ;
         O_oe_trdy <= 1'b1 ;
         O_oe_stop <= 1'b1 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;

         O_incr_twf_wrptr <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;
      end 

   ABORT :
      begin
         trdy_N <= 1'b1 ;
         stop_N <= 1'b0 ;
         O_oe_trdy <= 1'b1 ;
         O_oe_stop <= 1'b1 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;

         O_incr_twf_wrptr <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;
      end 

   LAST_CYC:
      begin 
         trdy_N <= 1'b1 ; // drive with deasserted values 
         stop_N <= 1'b1 ;

/*       O_oe_trdy <= 1'b1 ;
         O_oe_stop <= 1'b1 ;
*/

         O_oe_trdy <= 1'b0 ; // 23 Mar See explanation in devsel_sm block
         O_oe_stop <= 1'b0 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;

         O_incr_twf_wrptr <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;
      end 


   RD_WAIT,WR_WAIT: // added 12th March should verify 
  // when it is a wait for  data to be available, even though trdy is deasserted,
  // they have to be driven, so their enables should be high

      begin
         trdy_N <= 1'b1 ;
         stop_N <= 1'b1 ;
         O_oe_trdy <= 1'b1 ;
         O_oe_stop <= 1'b1 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;

         O_incr_twf_wrptr <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;
      end 

   default : 
      begin
         trdy_N <= 1'b1 ;
         stop_N <= 1'b1 ;
         O_oe_trdy <= 1'b0 ;
         O_oe_stop <= 1'b0 ;

         oe_AD_lo  <= 1'b0 ;
         oe_AD_hi  <= 1'b0 ;

         O_incr_twf_wrptr <= 1'b0 ;
         O_incr_trf_rdptr <= 1'b0 ;
      end 

   endcase
end


// generate read mode 
always@(present_state)
begin

   case (present_state) 
   
   RD_WAIT, RD_XFER,RD_INIT_LAT : O_read_mode <= 1'b1 ;

   default : O_read_mode <= 1'b0 ;
   endcase
end

// generate last_cycle_indi
always@(present_state)
begin
   case(present_state)
   LAST_CYC : last_cyc_indi <= 1'b1 ;
   default  : last_cyc_indi <= 1'b0 ;
   endcase
end 

// The parity output enables should follow the data enables by one clock cycle

always@(negedge I_RST_N or posedge I_CLK)
begin

if (I_RST_N == 1'b0) 

   begin 
      O_oe_PAR <= 1'b0 ;
      O_oe_PAR64 <= 1'b0 ;
   end
else
   begin 
      O_oe_PAR <= oe_AD_lo ;
      O_oe_PAR64 <= oe_AD_hi ;
   end
end  

endmodule  
